{-# LINE 32 "Dict.lhs" #-}
  module Algos.DS.Dict ( EqDict
                       , OrdDict
                       , IxDict
                       , makeEqDict
                       , makeOrdDict
                       , makeIxDict
                       , eqDictFind
                       , ordDictFind
                       , ixDictFind
                       ) where
 
  import Algos.DivideAndConquer.Sorting (mergeSortBy)
  import Data.Array (Array,accumArray,(!))
  import Data.Function (on)
  import Data.Ix (Ix)
  import Data.List (groupBy)
{-# LINE 85 "Dict.lhs" #-}
  newtype EqDict k v = EqDict [(k,v)]
 
  makeEqDict     ::  Eq k => [k] -> [(k,v)] -> EqDict k v
  makeEqDict  _  =   EqDict
 
  eqDictFind                ::  Eq k => EqDict k v -> k -> [v]
  eqDictFind (EqDict xs) k  =   map snd (filter (\(k', _) -> k == k') xs)
{-# LINE 106 "Dict.lhs" #-}
  data OrdDict k v   =  EmptyOrdDict
                     |  OrdDict  {  key                    ::  k
                                 ,  vals                   ::  [v]
                                 ,  leftChild, rightChild  ::  OrdDict k v
                                 }
 
  makeOrdDict        ::  Ord k => [k] -> [(k,v)] -> OrdDict k v
  makeOrdDict _  []  =   EmptyOrdDict
  makeOrdDict _  xs  =   mergeDicts                                                                   $ 
                         map (\xs -> OrdDict (fst $ head xs) (map snd xs) EmptyOrdDict EmptyOrdDict)  $ 
                         groupBy ((==) `on` fst)                                                      $
                         mergeSortBy (compare `on` fst) xs
 
    where  mergeDicts [d]  = d
           mergeDicts ds   = mergeDicts (mergePairs ds)
 
           mergePairs []                            = []
           mergePairs [d]                           = [d]
           mergePairs [d_1, d_2]                    = [d_2 { leftChild = d_1 }]
           mergePairs [d_1, d_2, d_3]               = [d_2 { leftChild = d_1, rightChild = d_3 }]
           mergePairs (d_1 : d_2 : d_3 : d_4 : ds)  = d_2 { leftChild = d_1, rightChild = d_3 } : d_4 : mergePairs ds
{-# LINE 148 "Dict.lhs" #-}
  ordDictFind      ::  Ord k => OrdDict k v -> k -> [v]
  ordDictFind d k  =   fd d
    where  fd  EmptyOrdDict         =  []
           fd  (OrdDict k' vs l r)  |  k == k'    =  vs
                                    |  k < k'     =  fd l
                                    |  otherwise  =  fd r
{-# LINE 163 "Dict.lhs" #-}
  newtype IxDict k v = IxDict (Array k [v])
 
  makeIxDict        ::  Ix k => [k] -> [(k,v)] -> IxDict k v
  makeIxDict ks xs  =   IxDict $ fmap reverse $ accumArray (flip (:)) [] (l,u) xs
    where  l  = minimum  ks
           u  = maximum  ks
{-# LINE 187 "Dict.lhs" #-}
  ixDictFind                ::  Ix k => IxDict k v -> k -> [v]
  ixDictFind  (IxDict a) k  =   a ! k
