{-# LINE 41 "Sorting.lhs" #-}
  {-#  LANGUAGE RankNTypes  #-}
 
  module Algos.DivideAndConquer.Sorting ( insertionSort
                                        , insertionSortBy
                                        , mergeSort
                                        , mergeSortBy
                                        , merge
                                        , mergeBy
                                        , quickSort
                                        , quickSortBy
                                        , simpleQuickSort
                                        , simpleQuickSortBy
                                        , worstCaseQuickSort
                                        , worstCaseQuickSortBy
                                        , randomPermutationQuickSort
                                        , randomPermutationQuickSortBy
                                        , randomPivotQuickSort
                                        , randomPivotQuickSortBy
                                        , select
                                        , selectBy
                                        , simpleSelect
                                        , simpleSelectBy
                                        , worstCaseSelect
                                        , worstCaseSelectBy
                                        , randomPermutationSelect
                                        , randomPermutationSelectBy
                                        , partition
                                        , partitionBy
                                        , randomPivotSelect
                                        , randomPivotSelectBy
                                        , bucketSort
                                        , bucketSortBy
                                        , countingSort
                                        , countingSortBy
                                        , radixSort
                                        , radixSortBy
                                        ) where
 
  import Algos.Random.Sequences (randomPermutation)
  import Control.Monad.State
  import Control.Monad.ST
  import Data.Array
  import Data.Array.ST
  import Data.List (unfoldr, foldl')
  import Data.Ord (comparing)
  import System.Random
{-# LINE 111 "Sorting.lhs" #-}
  insertionSort  ::  Ord a => [a] -> [a]
  insertionSort  =   insertionSortBy compare
 
  insertionSortBy         ::  (a -> a -> Ordering) -> [a] -> [a]
  insertionSortBy cmp xs  =   insSort xs
    where  insSort []        =  []
           insSort (x : xs)  =  ins (insSort xs)
             where  ins []            =  [x]
                    ins ys@(y : ys')  |  cmp x y == GT  =  y  :  ins ys'
                                      |  otherwise      =  x  :  ys
{-# LINE 141 "Sorting.lhs" #-}
  mergeSort   ::   Ord a => [a] -> [a]
  mergeSort   =    mergeSortBy compare
 
  mergeSortBy          ::  (a -> a -> Ordering) -> [a] -> [a]
  mergeSortBy cmp xs   =   mSort xs
    where  mSort  []  =  []
           mSort  xs  =  ms $ map (: []) xs
             where  ms [s]             =  s
                    ms ss              =  ms (m2 ss)
                    m2 (s1 : s2 : ss)  =  mergeBy cmp s1 s2 : m2 ss
                    m2 ss              =  ss
{-# LINE 163 "Sorting.lhs" #-}
  merge  ::  Ord a => [a] -> [a] -> [a]
  merge  =   mergeBy compare
 
  mergeBy            ::  (a -> a -> Ordering) -> [a] -> [a] -> [a]
  mergeBy cmp ls rs  =   mrg ls rs
    where  mrg  ls            []            =  ls
           mrg  []            rs            =  rs
           mrg  ls@(l : ls')  rs@(r : rs')  |  cmp l r == GT  =  r  :  mrg ls   rs'
                                            |  otherwise      =  l  :  mrg ls'  rs
{-# LINE 199 "Sorting.lhs" #-}
  newtype PivotSelect a = PS { runPS :: [a] -> State (PivotSelect a) a }
 
  getPivot     ::  [a] -> State (PivotSelect a) a
  getPivot xs  =   get >>= flip runPS xs
{-# LINE 206 "Sorting.lhs" #-}
  quickSort            ::  Ord a => PivotSelect a -> [a] -> [a]
  quickSort findPivot  =   quickSortBy findPivot compare
 
  quickSortBy                   ::  PivotSelect a -> (a -> a -> Ordering) -> [a] -> [a]
  quickSortBy findPivot cmp xs  =   evalState (qs xs) findPivot
    where  qs  []  =  return []
           qs  xs  =  do  p <- getPivot xs
                          let (ls, ms, rs) = partitionBy cmp p xs
                          sls  <- qs ls
                          srs  <- qs rs
                          return (sls ++ ms ++ srs)
{-# LINE 221 "Sorting.lhs" #-}
  partition   ::  Ord a => a -> [a] -> ([a], [a], [a])
  partition   =   partitionBy compare
 
  partitionBy        ::  (a -> a -> Ordering) -> a -> [a] -> ([a], [a], [a])
  partitionBy cmp p  =   part
    where  part  []        =  ([], [], [])
           part  (x : xs)  |  cmp x p == LT  =  (x :  ls,      ms,      rs)
                           |  cmp x p == EQ  =  (     ls, x :  ms,      rs)
                           |  otherwise      =  (     ls,      ms, x :  rs)
             where (ls, ms, rs) = part xs
{-# LINE 236 "Sorting.lhs" #-}
  simpleQuickSort  ::  Ord a => [a] -> [a]
  simpleQuickSort  =   simpleQuickSortBy compare
                        
  simpleQuickSortBy  ::  (a -> a -> Ordering) -> [a] -> [a]  
  simpleQuickSortBy  =   quickSortBy pickFirst
 
  pickFirst  ::  PivotSelect a
  pickFirst  =   PS (return . head)
{-# LINE 252 "Sorting.lhs" #-}
  worstCaseQuickSort  ::  Ord a => [a] -> [a]
  worstCaseQuickSort  =   worstCaseQuickSortBy compare
 
  worstCaseQuickSortBy       ::  (a -> a -> Ordering) -> [a] -> [a]
  worstCaseQuickSortBy  cmp  =   quickSortBy (pickMedianBy cmp) cmp
 
  pickMedianBy      ::  (a -> a -> Ordering) -> PivotSelect a
  pickMedianBy cmp  =   PS $ \xs -> return (worstCaseSelectBy cmp (length xs `div` 2) xs)
{-# LINE 271 "Sorting.lhs" #-}
  randomPermutationQuickSort  ::  (Ord a, RandomGen g) => g -> [a] -> [a]
  randomPermutationQuickSort  =   randomPermutationQuickSortBy compare
 
  randomPermutationQuickSortBy         ::  RandomGen g => (a -> a -> Ordering) -> g -> [a] -> [a]
  randomPermutationQuickSortBy cmp g   =   simpleQuickSortBy cmp . randomPermutation g
{-# LINE 291 "Sorting.lhs" #-}
  randomPivotQuickSort  ::  (Ord a, RandomGen g) => g -> [a] -> [a]
  randomPivotQuickSort  =   randomPivotQuickSortBy compare
 
  randomPivotQuickSortBy        ::  RandomGen g => (a -> a -> Ordering) -> g -> [a] -> [a]
  randomPivotQuickSortBy cmp g  =   quickSortBy (pickRandom g) cmp
 
  pickRandom    ::  RandomGen g => g -> PivotSelect a
  pickRandom g  =   PS $ \xs ->  do  let (i, g') = randomR (0, length xs - 1) g
                                     put (pickRandom g')
                                     return (xs !! i)
{-# LINE 308 "Sorting.lhs" #-}
  select            ::  Ord a => PivotSelect a -> Int -> [a] -> a
  select findPivot  =   selectBy findPivot compare
{-# LINE 313 "Sorting.lhs" #-}
  selectBy                     ::   PivotSelect a -> (a -> a -> Ordering) -> Int -> [a] -> a
  selectBy findPivot cmp k xs  =    evalState (sel k xs) findPivot
    where sel k xs = do  p <- getPivot xs
                         let  (ls, ms, rs)  =  partitionBy cmp p xs
                              nls           =  length ls
                              nms           =  length ms
                         if k < nls
                           then  sel k ls
                           else  if k < nls + nms
                                   then  return (head ms)
                                   else  sel (k - nls - nms) rs
{-# LINE 328 "Sorting.lhs" #-}
  simpleSelect  ::  Ord a => Int -> [a] -> a
  simpleSelect  =   simpleSelectBy compare
 
  simpleSelectBy  ::  (a -> a -> Ordering) -> Int -> [a] -> a
  simpleSelectBy  =   selectBy pickFirst
 
  randomPermutationSelect  ::  (Ord a, RandomGen g) => g -> Int -> [a] -> a
  randomPermutationSelect  =   randomPermutationSelectBy compare
 
  randomPermutationSelectBy          ::  RandomGen g => (a -> a -> Ordering) -> g -> Int -> [a] -> a
  randomPermutationSelectBy cmp g k  =   simpleSelectBy cmp k . randomPermutation g
 
  randomPivotSelect  ::  (Ord a, RandomGen g) => g -> Int -> [a] -> a
  randomPivotSelect  =   randomPivotSelectBy compare
 
  randomPivotSelectBy        ::  RandomGen g => (a -> a -> Ordering) -> g -> Int -> [a] -> a
  randomPivotSelectBy cmp g  =   selectBy (pickRandom g) cmp
{-# LINE 352 "Sorting.lhs" #-}
  worstCaseSelect  ::  Ord a => Int -> [a] -> a
  worstCaseSelect  =   worstCaseSelectBy compare
 
  worstCaseSelectBy      ::  (a -> a -> Ordering) -> Int -> [a] -> a
  worstCaseSelectBy cmp  =   selectBy (pickApproxMedianBy cmp) cmp
{-# LINE 360 "Sorting.lhs" #-}
  pickApproxMedianBy      ::  (a -> a -> Ordering) -> PivotSelect a
  pickApproxMedianBy cmp  =   PS $ \xs -> return (go xs)
    where   go  [x]  =  x
            go  xs   =  worstCaseSelectBy cmp (length ys `div` 2) ys
              where ys = fiveSampleBy cmp xs
 
  fiveSampleBy      ::   (a -> a -> Ordering) -> [a] -> [a]
  fiveSampleBy cmp  =    unfoldr pickSample
    where  pickSample  []  =   Nothing
           pickSample  xs  =   Just (m, zs)
             where    (ys, zs)  =  splitAt 5 xs
                      m         =  sortSelectBy cmp (length ys `div` 2) ys
{-# LINE 376 "Sorting.lhs" #-}
  sortSelectBy           ::  (a -> a -> Ordering) -> Int -> [a] -> a
  sortSelectBy cmp k xs  =   insertionSortBy cmp xs !! k
{-# LINE 384 "Sorting.lhs" #-}
  bucketSort  ::  Real t => [t] -> [t]
  bucketSort  =   bucketSortBy realToFrac
 
  bucketSortBy         ::  (a -> Double) -> [a] -> [a]
  bucketSortBy key xs  =   concatMap (insertionSortBy $ comparing key) (elems a)
    where  n           =  length  xs
           keys        =  map key xs
           minKey      =  minimum keys
           maxKey      =  maximum keys
           scale       =  fromIntegral n / (maxKey - minKey)
           buckets     =  map (\k -> floor ((k - minKey) * scale)) keys
           bucketvals  =  zip buckets xs
           a           =  fmap reverse $ accumArray (flip (:)) [] (0, n) bucketvals
{-# LINE 421 "Sorting.lhs" #-}
  countingSortBy :: (a -> Int) -> [a] -> [a]
{-# LINE 431 "Sorting.lhs" #-}
  countingSort  ::  Integral t => [t] -> [t]
  countingSort  =   countingSortBy fromIntegral
{-# LINE 436 "Sorting.lhs" #-}
  countingSortBy  _    []  =  []
  countingSortBy  key  xs  =  concat $ elems a
    where  keys     =   map  key   xs
           keyvals  =   zip  keys  xs
           minKey   =   minimum  keys
           maxKey   =   maximum  keys
           a        =   fmap reverse $ accumArray (flip (:)) [] (minKey, maxKey) keyvals
{-# LINE 463 "Sorting.lhs" #-}
  radixSortBy :: [a -> Int] -> [a] -> [a]
{-# LINE 472 "Sorting.lhs" #-}
  radixSort      ::  Integral t => [t] -> [t]
  radixSort []   =   []
  radixSort [x]  =   [x]
  radixSort xs   =   radixSortBy digits xs
    where  n           =  length xs
           maxx        =  maximum xs
           k           =  1 + ilog n (fromIntegral maxx)
           digits      =  map digit (take k divs)
           divs        =  1 : map (* n) divs
           digit dv x  =  (fromIntegral x `div` dv) `mod` n
 
           ilog base x = go 0 x
             where  go lg y  |  y < base   =  lg
                             |  otherwise  =  go (lg+1) (y `div` n)
{-# LINE 502 "Sorting.lhs" #-}
  radixSortBy keys xs = foldl' (flip countingSortBy) xs keys
