{-------------------------------------------------------------------------------

Common code for the DFS and BFS wrappers

-------------------------------------------------------------------------------}

module Algos.Graphs.DFSBFSMain ( commonMain
                               ) where

import Algos.Graphs.Forest (Forest(..), Tree(..), Subtree(..))
import Algos.Graphs.Graph (Graph, AdjList(..), V, E, makeDirAdjListIx, makeUndirAdjListIx)
import Algos.Graphs.GraphIO (Display(..), Parse(..), parseGraph, displayNewick, Orientation(..))
import Control.Applicative ((<$>))
import System.Environment (getProgName, getArgs)
import System.Exit (exitFailure)
import System.IO (withFile, IOMode(..), hPutStr)

-- The main function
commonMain :: (AdjList Int Int Int -> Forest V E) -> IO ()
commonMain traversal = 
  do (gfile, ffile) <- readArgs
     g <- parseGraph <$> readFile gfile
     let f = runTraversal traversal g
     withFile ffile WriteMode $ \h -> hPutStr h (displayNewick f)
            
readArgs :: IO (String, String)
readArgs =  do args <- getArgs
               case args of
                 [gfile, ffile] -> return (gfile, ffile)
                 _              -> usage >> exitFailure

-- Prints a usage message in case the command line arguments make no sense
usage :: IO ()
usage = do prog <- getProgName
           putStrLn $ "USAGE: " ++ prog ++ " <graph file> <forest file>"
           
-- Runs the traversal
runTraversal                  :: (AdjList Int Int Int -> Forest V E) ->
                                 (Orientation, Graph Int Int Int) -> Forest Int Int
runTraversal traversal (o, g) =  f
  where a  | o == Directed   = makeDirAdjListIx   g
           | o == Undirected = makeUndirAdjListIx g
        f' = traversal a
        f  = relabelForest a f'

-- Replaces the vertices and edges in a forest with their IDs and labels,
-- respectively.
relabelForest               :: AdjList v vl el -> Forest V E -> Forest v el
relabelForest a (Forest ts) =  Forest (map (relabelTree a) ts)

-- Replaces the vertices and edges in a tree with their IDs and labels,
-- respectively.
relabelTree               :: AdjList v vl el -> Tree V E -> Tree v el
relabelTree a (Tree v ws) =  Tree (adjVertexId a v) (map (relabelSubtree a) ws)

-- Replaces the vertices and edges in a subtree with their IDs and labels,
-- respectively.
relabelSubtree                 :: AdjList v vl el -> Subtree V E -> Subtree v el
relabelSubtree a (Subtree e t) =  Subtree (adjEdgeLabel a e) (relabelTree a t)

-- A parse instance for strings that doesn't require quotation marks
instance Parse Int where
  parse = read

-- A display instance for strings that doesn't print quotation marks
instance Display Int where
  display = show
