{-# LINE 32 "Graph.lhs" #-}
  module Algos.Graphs.Graph ( Graph(..)
                            , Vertex(..)
                            , Edge(..)
                            , AdjList(..)
                            , GraphStructure(..)
                            , V(..)
                            , E(..)
                            , makeDirAdjListEq
                            , makeDirAdjListOrd
                            , makeDirAdjListIx
                            , makeUndirAdjListEq
                            , makeUndirAdjListOrd
                            , makeUndirAdjListIx
                            , makeDirAdjListIndexByEq
                            , makeDirAdjListIndexByOrd
                            , makeDirAdjListIndexByIx
                            , makeUndirAdjListIndexByEq
                            , makeUndirAdjListIndexByOrd
                            , makeUndirAdjListIndexByIx
                            , makeDirGraph
                            , makeUndirGraph
                            , vIncidentEdges
                            , vInNeighbours
                            , vOutNeighbours
                            , vNeighbours
                            , vInDegree
                            , vOutDegree
                            , vDegree
                            , makeVAttr
                            , makeEAttr
                            , flipGraph
                            ) where
 
  import Algos.DS.Dict
  import Data.Array (Array,array,listArray,(!),elems)
  import Data.Ix (Ix)
{-# LINE 125 "Graph.lhs" #-}
  data Graph v vl el = Graph  {  graphVertices  ::  [Vertex v vl]
                              ,  graphEdges     ::  [Edge   v el]
                              }
 
  data Vertex v vl = Vertex  {  vertexId     ::  v
                             ,  vertexLabel  ::  vl
                             }
 
  data Edge v el = Edge  {  edgeTail, edgeHead  ::  v
                         ,  edgeLabel           ::  el
                         }
{-# LINE 140 "Graph.lhs" #-}
  data AdjList v vl el = AdjList  {  adjGraph        ::  GraphStructure
                                  ,  adjVertexId     ::  V  ->  v
                                  ,  adjVertexLabel  ::  V  ->  vl
                                  ,  adjEdgeLabel    ::  E  ->  el
                                  }
{-# LINE 154 "Graph.lhs" #-}
  data GraphStructure = G  {  gNumVertices  ::  Int
                           ,  gNumEdges     ::  Int
                           ,  gVertices     ::  [V]
                           ,  gEdges        ::  [E]  
                           }
{-# LINE 164 "Graph.lhs" #-}
  data V = V  {  vIx        ::  Int
              ,  vInEdges   ::  [E]
              ,  vOutEdges  ::  [E]  
              }
{-# LINE 171 "Graph.lhs" #-}
  data E = E  {  eIx           ::  Int
              ,  eTail, eHead  ::  V
              }
{-# LINE 200 "Graph.lhs" #-}
  makeDirAdjListEq     ::  Eq v   => Graph v vl el -> AdjList v vl el
  makeDirAdjListOrd    ::  Ord v  => Graph v vl el -> AdjList v vl el
  makeDirAdjListIx     ::  Ix v   => Graph v vl el -> AdjList v vl el
  makeUndirAdjListEq   ::  Eq v   => Graph v vl el -> AdjList v vl el
  makeUndirAdjListOrd  ::  Ord v  => Graph v vl el -> AdjList v vl el
  makeUndirAdjListIx   ::  Ix v   => Graph v vl el -> AdjList v vl el
{-# LINE 209 "Graph.lhs" #-}
  makeDirAdjListIndexByEq     ::  Eq t   => (v -> t) -> Graph v vl el -> AdjList v vl el
  makeDirAdjListIndexByOrd    ::  Ord t  => (v -> t) -> Graph v vl el -> AdjList v vl el
  makeDirAdjListIndexByIx     ::  Ix t   => (v -> t) -> Graph v vl el -> AdjList v vl el
  makeUndirAdjListIndexByEq   ::  Eq t   => (v -> t) -> Graph v vl el -> AdjList v vl el
  makeUndirAdjListIndexByOrd  ::  Ord t  => (v -> t) -> Graph v vl el -> AdjList v vl el
  makeUndirAdjListIndexByIx   ::  Ix t   => (v -> t) -> Graph v vl el -> AdjList v vl el
{-# LINE 218 "Graph.lhs" #-}
  makeDirAdjListEq     = makeDirAdjListIndexByEq     id
  makeDirAdjListOrd    = makeDirAdjListIndexByOrd    id
  makeDirAdjListIx     = makeDirAdjListIndexByIx     id
  makeUndirAdjListEq   = makeUndirAdjListIndexByEq   id
  makeUndirAdjListOrd  = makeUndirAdjListIndexByOrd  id
  makeUndirAdjListIx   = makeUndirAdjListIndexByIx   id
{-# LINE 227 "Graph.lhs" #-}
  makeUndirAdjListIndexByEq   f = makeDirAdjListIndexByEq   f . dupEdges
  makeUndirAdjListIndexByOrd  f = makeDirAdjListIndexByOrd  f . dupEdges
  makeUndirAdjListIndexByIx   f = makeDirAdjListIndexByIx   f . dupEdges
 
  dupEdges    ::  Graph v vl el -> Graph v vl el
  dupEdges g  =   g { graphEdges = concatMap (\e@(Edge u v l) -> [e, Edge v u l]) (graphEdges g) }
{-# LINE 237 "Graph.lhs" #-}
  data Indices = Indices  {  inIxs    ::  [[Int]]
                          ,  outIxs   ::  [[Int]]
                          ,  tailIxs  ::  [Int]
                          ,  headIxs  ::  [Int]  
                          }
{-# LINE 252 "Graph.lhs" #-}
  makeAdjList         ::  Graph v vl el -> Indices -> AdjList v vl el
  makeAdjList g ixs   =   AdjList   {   adjGraph        =  gs
                                    ,   adjVertexId     =  vid
                                    ,   adjVertexLabel  =  vlab
                                    ,   adjEdgeLabel    =  elab
                                    }
 
    where  n     =  length (graphVertices g)
           m     =  length (graphEdges g)
           vid   =  makeVAttr $ zip [1..] (map vertexId     $ graphVertices g)
           vlab  =  makeVAttr $ zip [1..] (map vertexLabel  $ graphVertices g)
           elab  =  makeEAttr $ zip [1..] (map edgeLabel    $ graphEdges g)
 
           gs = G  {   gNumVertices   =  n
                   ,   gNumEdges      =  m
                   ,   gVertices      =  vs
                   ,   gEdges         =  es 
                   }
 
           vs  =  zipWith3  makeV  [1..] (inIxs ixs) (outIxs ixs)
           es  =  zipWith3  makeE  [1..] (tailIxs ixs) (headIxs ixs)
           va  =  listArray   (1,n)  vs
           ea  =  listArray   (1,m)  es
 
           makeV i is os   = V i (map (ea !) is) (map (ea !) os)
           makeE i t h     = E i (va ! t) (va ! h)
{-# LINE 287 "Graph.lhs" #-}
  makeVAttr     ::   [(Int,t)] -> V -> t
  makeVAttr as  =    vAttr   
    where  a        =  array (1, length as) as                    
           vAttr v  =  a ! vIx v
 
  makeEAttr     ::   [(Int,t)] -> E -> t
  makeEAttr as  =    eAttr
    where  a        =  array (1, length as) as
           eAttr e  =  a ! eIx e
{-# LINE 323 "Graph.lhs" #-}
  makeIndices                         ::   (v -> k) -> ([k] -> [(k,Int)] -> d) -> (d -> k -> [Int]) -> 
                                           Graph v vl el -> Indices
  makeIndices f makeDict findDict g   =    Indices is os ts hs
 
    where  vertexKeys  =  map (f . vertexId)  (graphVertices g)
           tailKeys    =  map (f . edgeTail)  (graphEdges g)
           headKeys    =  map (f . edgeHead)  (graphEdges g)
 
           indexedVs       =  makeDict vertexKeys  (zip vertexKeys  [1..])
           tailIndexedEs   =  makeDict vertexKeys  (zip tailKeys    [1..])
           headIndexedEs   =  makeDict vertexKeys  (zip headKeys    [1..])
 
           is  =  map (        findDict headIndexedEs  . f . vertexId)  (graphVertices g)
           os  =  map (        findDict tailIndexedEs  . f . vertexId)  (graphVertices g)
           ts  =  map (head .  findDict indexedVs      . f . edgeTail)  (graphEdges g)
           hs  =  map (head .  findDict indexedVs      . f . edgeHead)  (graphEdges g)
{-# LINE 343 "Graph.lhs" #-}
  makeDirAdjListIndexByEq f g = makeAdjList g (makeIndices f makeEqDict eqDictFind g)
{-# LINE 355 "Graph.lhs" #-}
  makeDirAdjListIndexByOrd f g = makeAdjList g (makeIndices f makeOrdDict ordDictFind g)
{-# LINE 372 "Graph.lhs" #-}
  makeDirAdjListIndexByIx f g = makeAdjList g (makeIndices f makeIxDict ixDictFind g)
{-# LINE 384 "Graph.lhs" #-}
  makeDirGraph  ::  AdjList v vl el -> Graph v vl el
  makeDirGraph  =   makeGraph (const True)
 
  makeUndirGraph   ::  AdjList v vl el -> Graph v vl el
  makeUndirGraph   =   makeGraph (\(E _ t h) -> vIx t < vIx h)
 
  makeGraph                                     ::  (E -> Bool) -> AdjList v vl el -> Graph v vl el
  makeGraph edgeFilt (AdjList g vid vlab elab)  =   Graph vs es
    where  vs  =  map (\v            -> Vertex  (vid v)          (vlab v))  (gVertices g)
           es  =  map (\e@(E _ t h)  -> Edge    (vid t) (vid h)  (elab e))  (filter edgeFilt $ gEdges g)
{-# LINE 400 "Graph.lhs" #-}
  vIncidentEdges    ::  V -> [E]
  vIncidentEdges v  =   vInEdges v ++ vOutEdges v
 
  vInNeighbours, vOutNeighbours, vNeighbours :: V -> [V]
  vInNeighbours   =  map eTail . vInEdges
  vOutNeighbours  =  map eHead . vOutEdges
  vNeighbours v   =  vInNeighbours v ++ vOutNeighbours v
{-# LINE 410 "Graph.lhs" #-}
  vInDegree, vOutDegree, vDegree :: V -> Int
  vInDegree   =  length . vInEdges
  vOutDegree  =  length . vOutEdges
  vDegree v   =  vInDegree v + vOutDegree v
{-# LINE 428 "Graph.lhs" #-}
  flipGraph    ::  AdjList v vl el -> AdjList v vl el
  flipGraph g  =   g { adjGraph = flippedGS }
    where  gs         =  adjGraph g
           flippedGS  =  gs  {  gVertices  =  map ((!) vs . vIx) (gVertices gs)
                             ,  gEdges     =  map ((!) es . eIx) (gEdges gs)
                             }
           vs         =  array (1,gNumVertices gs)   $ map makeV  (gVertices  gs)
           es         =  array (1,gNumEdges gs)      $ map makeE  (gEdges gs)
           makeV v    =  ( vIx v, v   {  vInEdges   =  map (\e -> es ! eIx e) (vOutEdges v)
                                      ,  vOutEdges  =  map (\e -> es ! eIx e) (vInEdges v)
                                      } )
           makeE e    =  ( eIx e, e  {  eTail  =  vs ! (vIx $ eHead e)
                                     ,  eHead  =  vs ! (vIx $ eTail e)
                                     } )
