{-# LANGUAGE TupleSections #-}

module Algos.Random.Impl ( randomNumberM
                         , floydM
                         ) where

import Control.Applicative ((<$>))
import Control.Monad.State (State, state)
import Data.List (foldl')
import Data.Map (Map, empty, (!))
import System.Random (RandomGen, randomR)

type AddFun = ([Int], Map Int [Int]) -> (Int, Int) -> ([Int], Map Int [Int])
type SeqFun = ([Int], Map Int [Int]) -> ([Int], Map Int [Int])

-- A State monad version of System.Random.randomR
randomNumberM :: RandomGen g => (Int, Int) -> State g Int
randomNumberM =  state . randomR

-- The core of randomCombinationM and randomSubPermutationM.  They are both
-- variants of Floyd's algorithm for generating uniformly random permutations
-- and combinations.
floydM :: RandomGen g => AddFun -> SeqFun -> Int -> Int -> State g [Int]
floydM addNum toSeq p n = flattenSeq . toSeq . foldl' addNum ([], empty) <$> 
                          mapM (\x -> (,x) <$> randomNumberM (1,x)) [n-p'+1..n]
  where p' = min p n
        flattenSeq (rs, m) = fseq rs []
          where fseq []     seq = seq
                fseq (x:xs) seq = x:fseq (m!x) (fseq xs seq)
