module Algos.Random.Sets ( randomCombination
                         , randomCombination'
                         , randomCombinationM
                         ) where

import Algos.Random.Impl (floydM)
import Control.Monad.State (State, runState)
import Data.Map (member, insert, keys)
import System.Random (RandomGen)

-- Computes a random combination (in sorted order) of p integers between 1 and n
-- in O(p lg p) time.
randomCombination       :: RandomGen g => g -> Int -> Int -> [Int]
randomCombination g p n =  fst $ randomCombination' g p n

-- A version of randomCombination that threads through the random number generator
randomCombination'       :: RandomGen g => g -> Int -> Int -> ([Int], g)
randomCombination' g p n =  runState (randomCombinationM p n) g

-- A State monad version of randomCombination'
randomCombinationM :: RandomGen g => Int -> Int -> State g [Int]
randomCombinationM =  floydM addNum toSeq
  where addNum (l, m) (t, j) | member t m = (l, insert j [] m)
                             | otherwise  = (l, insert t [] m)
        toSeq (_, m)                      = (keys m, m)
